/*
 *  Debug watch point handling
 */

#ifdef __DJGPP__
#error djgpp not yet supported!
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#define __EXC_INTERNAL
#include "exc.h"
#include "watch.h"
#include "printk.h"

struct Watch  {
       struct Watch *next;
       ULONG  addr;
       ULONG  type;
       UCHAR  length;
       UCHAR  DRx;
     };

static struct Watch *watch0 = NULL;
static int    cur_reg;                 /* DR0-3 to use */

static int set_watch   (struct Watch *w, int set);
static void align_addr (ULONG *addr, int len);

/*
 *  Set or modify an existing watch-point.
 *  Returns : 0 = OK
 *   else Error, overlapping watch-point
 *    "   Error, illegal address
 *    "   Error, illegal length
 *    "   Errorm illegal break type
 *    "   Error, cannot expand watch-list
 */
int SetDebugWatch (ULONG addr, int length, ULONG type)
{
  struct Watch *w;

  if (addr < excAppStart || (addr + length) >= excStkStart)
     return (WP_ERR_ILL_ADDR);

  if (length != 1 && length != 2 && length != 4)
     return (WP_ERR_ILL_LENGTH);

  if (type != DR7_BINST && type != DR7_BWR && type != DR7_BRW)
     return (WP_ERR_ILL_TYPE);

  align_addr (&addr,length);

  for (w = watch0; w; w = w->next)       /* Overlapping areas:             */
  {                                      /*                                */
    if ((w->addr + w->length > addr) &&  /*      ******  addr,length       */
        (w->addr < addr))                /*    ****      w->addr,w->length */
       return (WP_ERR_OVERLAP);          /*                                */
                                         /*                                */
    if ((addr + length > w->addr) &&     /*   ******     addr,length       */
        (w->addr > addr))                /*       *****  w->addr,w->length */
       return (WP_ERR_OVERLAP);          /*                                */

    /*
     * Pass through these combinations (grow/shrink area)
     *
     *  ******           ****
     *  ****             ********
     */ 
    if (w->addr == addr)
       break;
  }

  if (!w)
  {
    w = malloc (sizeof(*w));
    if (!w)
       return (WP_ERR_NO_MEM);

    w->next = watch0;
    w->DRx  = cur_reg;
    cur_reg = ++cur_reg & 3;
    watch0  = w;
  }
  w->addr   = addr;
  w->length = length;
  w->type   = type;
  return set_watch (w,1);
}


/*
 *  Clear an existing watch-point.
 *  Returns : 0 = OK
 *    else Error, not found
 *     "   Error, illegal address
 */
int ClearDebugWatch (ULONG addr)
{
  struct Watch *w, *last;

  if (addr < excAppStart || addr >= excStkStart)
     return (WP_ERR_ILL_ADDR);

  for (w = last = watch0; w; last = w, w = w->next)
  {
    if ((w->addr - w->length <= addr) &&
        (w->addr + w->length >= addr))
    {
      int rc;
      if (w == watch0)
           watch0     = w->next;
      else last->next = w->next;
      rc = set_watch (w,0);
      free (w);
      return (rc);
    }
  }
  return (WP_ERR_NOT_FOUND);
}

/*
 *  Return a string describing the watch-point triggered
 *  at address `addr'
 */
char *GetDebugWatch (ULONG addr)
{
  SYS_REGS     reg;
  struct Watch *w;
  char         *ret = NULL;

  _dx_sysregs_get (&reg);
  printk ("DR6 = %08X\r\n", reg.dr6);

  reg.dr6 = 0;
  _dx_sysregs_set (&reg);

  if (addr < excAppStart || addr >= excStkStart)
     return (NULL);

  for (w = watch0; w; w = w->next)
     if (w->addr == addr)
     {
       static char buf[100];
       switch (w->type)
       {
         case DR7_BINST:
              sprintf (buf,"Exec at %s (len=%d)",
                       AddressSymbol(0,addr), w->length);
              break;
         case DR7_BWR:
              sprintf (buf,"Write at %s (len=%d)",
                       AddressSymbol(0,addr), w->length);
              break;
         case DR7_BRW:
              sprintf (buf,"Read at %s (len=%d)",
                       AddressSymbol(0,addr), w->length);
              break;
       }
       ret = buf;
       break;
     }

  ClearDebugWatch (addr);
  return (ret);
}

static int set_watch (struct Watch *w, int set)
{
  SYS_REGS reg;
  unsigned dr7_shift, dr7_mask = 0;

  _dx_sysregs_get (&reg);

  switch (w->DRx)
  {
    case 0: reg.dr0  = w->addr;
            dr7_mask = 0x000F0000;
            break;
    case 1: reg.dr1  = w->addr;
            dr7_mask = 0x00F00000;
            break;
    case 2: reg.dr2  = w->addr;
            dr7_mask = 0x0F000000;
            break;
    case 3: reg.dr3  = w->addr;
            dr7_mask = 0xF0000000;
            break;
  }
  if (w->type == DR7_BINST)
      w->length = 0;

  dr7_shift = DR7_RWLSHIFT (w->DRx);

  reg.dr7  = ~dr7_mask | ((w->type | 4 * w->length) << dr7_shift);
  reg.dr7 |= DR7_LE;                            /* Local Enable bit */
  if (set)
       reg.dr7 |= (1 << DR7_GLSHIFT (w->DRx));  /* Local Enable DRx */
  else reg.dr7 ^= (3 << DR7_GLSHIFT (w->DRx));  /* local/global disable */

  _dx_sysregs_set (&reg);
  return (WP_ERR_NONE);
}


static void align_addr (ULONG *addr, int len)
{
  if (len == 2) *addr &= ~1;
  if (len == 4) *addr &= ~3;
}
